--Modified by WrathOf to workaround enable texture background for containers versus the standard animation
--Use: In container prefab, add 2 fields to the class:
-- widgetbgimage = "fullpath/image.tex" and widgetbgimagepos = {x,y,z}
--
-- v1.1
-- Added support for inventory slot image tinting
--
--Updated for Revision 74452 2013-03-12_14-18-42 (A Little Rain Must Fall)
print("Loading containerwidget override")


require "class"
require "widgets/inventoryslot"
require "widgets/common"
local DOUBLECLICKTIME = .33

ContainerWidget = Class(Widget, function(self, owner)
    Widget._ctor(self, "Container")
    self:SetScale(.65,.65,.65)
    self.open = false
    self.inv = {}
    self.owner = owner
    self:SetHAnchor(ANCHOR_MIDDLE)
    self:SetVAnchor(ANCHOR_MIDDLE)
    self:SetPosition(0, 200, 0)
    self.slotsperrow = 3
    
    self.bg = self:AddChild(UIAnim())
    self.isopen = false

-- MOD --
	--Add support for image background as alternative to an animation
	self.bgimage = self:AddChild(Image())
	self.useimage = false
-- /MOD --
end)

function ContainerWidget:Open(container)
    self:Close()

-- MOD --    
	--Add support for background image
	if container.components.container.widgetbgimage then
		self.bgimage:SetTexture( container.components.container.widgetbgimage )
		self.bgimage:SetPosition( container.components.container.widgetbgimagepos )
		self.useimage = true
	end
-- /MOD --

    if container.components.container.widgetanimbank then
		self.bg:GetAnimState():SetBank(container.components.container.widgetanimbank)
	end
    
    if container.components.container.widgetanimbuild then
		self.bg:GetAnimState():SetBuild(container.components.container.widgetanimbuild)
    end
    
    if container.components.container.widgetpos then
		self:SetPosition(container.components.container.widgetpos)
	end
	
	if container.components.container.widgetbuttoninfo then
		self.button = self:AddChild(AnimButton("button_small"))
	    self.button:SetPosition(container.components.container.widgetbuttoninfo.position)
	    self.button:SetText(container.components.container.widgetbuttoninfo.text)
	    self.button:SetOnClick( function() container.components.container.widgetbuttoninfo.fn(container) end )
	    self.button:SetFont(BUTTONFONT)
	    self.button:SetTextSize(35)
	    self.button.text:SetVAlign(ANCHOR_MIDDLE)
	    self.button.text:SetColour(0,0,0,1)
	    
		if container.components.container.widgetbuttoninfo.validfn then
			if container.components.container.widgetbuttoninfo.validfn(container) then
				self.button:Enable()
			else
				self.button:Disable()
			end
		end
	end
	
	
    self.isopen = true
    self:Show()

-- MOD --    
	--Use background animation of texture not assigned
	if self.useimage then
		self.bgimage:Show()
	else
		self.bg:GetAnimState():PlayAnimation("open")
	end
-- /MOD --	    
    
    self.inst:ListenForEvent("itemlose", function(inst, data) self:OnItemLose(data.slot) end, container)
    self.inst:ListenForEvent("itemget", function(inst, data) self:OnItemGet(data.item, data.slot, data.src_pos) end, container)
	
	local num_slots = math.min( container.components.container:GetNumSlots(), #container.components.container.widgetslotpos)
	
	local n = 1
	for k,v in ipairs(container.components.container.widgetslotpos) do
	
		local slot = InvSlot(n,"data/images/inv_slot.tex", self.owner, container.components.container)
		self.inv[n] = self:AddChild(slot)

		slot:SetPosition(v)

-- MOD --
		if container.components.container.widgetbgimagetint then
			slot.bgimage:SetTint(
				container.components.container.widgetbgimagetint.r,
				container.components.container.widgetbgimagetint.g,
				container.components.container.widgetbgimagetint.b,
				container.components.container.widgetbgimagetint.a
			)
		end
-- /MOD --

		slot:SetLeftMouseDown(function() self:ClickInvSlot(slot) end)
		slot:SetRightMouseDown(function() self:RightClickInvSlot(slot) end)
		
		local obj = container.components.container:GetItemInSlot(n)
		if obj then
			local tile = ItemTile(obj, self)
			slot:SetTile(tile)
		end
		
		n = n + 1
	end

    self.container = container
    
end    


function ContainerWidget:RightClickInvSlot(slot)
	if self.owner and self.owner.components.inventory then
		local item = self.container.components.container:GetItemInSlot(slot.num)
		if item then
			if item.components.equippable then
				item = self.container.components.container:RemoveItem(item, true)
				self.owner.components.inventory:Equip(item)
	        else
				local actions = self.owner.components.playeractionpicker:GetInventoryActions(item, true)
				if actions then
	                self.owner.components.locomotor:PushAction(actions[1], true)
				end
			end
	    end
	end
end

function ContainerWidget:ClickInvSlot(slot)
	HandleContainerUIClick(self.owner, self.owner.components.inventory, self.container.components.container, slot.num)
end

function ContainerWidget:OnItemGet(item, slot , source_pos)
	
    if slot and self.inv[slot] then
		local tile = ItemTile(item, self)
        self.inv[slot]:SetTile(tile)
        tile:Hide()

        if source_pos then
			local dest_pos = self.inv[slot]:GetWorldPosition()
			local im = Image(item.components.inventoryitem:GetImage())
			im:MoveTo(source_pos, dest_pos, .3, function() tile:Show() tile:ScaleTo(2, 1, .25) im:Kill() end)
        else
			tile:Show() 
			tile:ScaleTo(2, 1, .25)
        end
        
	end
	
	
	if self.container and self.container.components.container.widgetbuttoninfo and self.container.components.container.widgetbuttoninfo.validfn then
		if self.container.components.container.widgetbuttoninfo.validfn(self.container) then
			self.button:Enable()
		else
			self.button:Disable()
		end
	end
	
	
end

function ContainerWidget:Update(dt)
	if self.isopen and self.owner and self.container then
		
		if not (self.container.components.inventoryitem and self.container.components.inventoryitem:IsHeldBy(self.owner)) then
			local distsq = self.owner:GetDistanceSqToInst(self.container)
			if distsq > 3*3 then
				self:Close()
			end
		end
	end
	
	return self.should_close_widget ~= true
end

function ContainerWidget:OnItemLose(slot)
	local tileslot = self.inv[slot]
	if tileslot then
		tileslot:SetTile(nil)
	end
	
	if self.container and self.container.components.container.widgetbuttoninfo and self.container.components.container.widgetbuttoninfo.validfn then
		if self.container.components.container.widgetbuttoninfo.validfn(self.container) then
			self.button:Enable()
		else
			self.button:Disable()
		end
	end
	
end


function ContainerWidget:Close()
    if self.isopen then
		
		if self.button then
			self.button:Kill()
			self.button = nil
		end
		
		if self.container then
			self.container.components.container:Close()
			--self.inst:RemoveAllEventCallbacks()
			self.inst:RemoveEventCallback("itemlose", self.container)
			self.inst:RemoveEventCallback("itemget", self.container)
		end
		
	    
		for k,v in pairs(self.inv) do
--			self:RemoveChild(v)
			v:Kill()
		end

		self.inv = {}

-- MOD --	    
		--Use animation of image not set
		if self.useimage then
			self.bgimage:Hide()
		else
			self.bg:GetAnimState():PlayAnimation("close")
		end
-- /MOD --

		self.container = nil
		
		self.isopen = false
		
	    self.inst:DoTaskInTime(.3, function()self.should_close_widget = true  end)
		
	end
    --self:Hide()

end